@@ -72,8 +72,20 @@
   :type 'integer
   :safe #'integerp)
 
+(defface a68-operator-face '((t :weight bold :foreground "white"))
+  "Face for printing Algol 68 operators.")
+
 (defface a68-string-break-face '((t :inherit font-lock-string-face))
-  "Face for printing Algol 64 string breaks.")
+  "Face for printing Algol 68 string breaks.")
+
+(defface a68-bits-radix-face '((t :weight bold))
+  "Face for printing the radix of Algol 68 bits denotations.")
+
+(defface a68-bits-flip-face '((t :weight bold))
+  "Face for printing set bits in binary Algol 68 bits denotations.")
+
+(defface a68-numbers-face '((t :weight bold :foreground "red"))
+  "Face for printing integer and real Algol 68 denotations")
 
 (defvar a68-mode-map
   (let ((map (make-sparse-keymap)))
@@ -99,15 +111,29 @@
   "Default comment style used by e.g. `comment-dwim'."
   :type '(choice (const "#")
                  (const "CO")
-                 (const "COMMENT"))
+                 (const "COMMENT")
+                 (const "NOTE")
+                 (const "{"))
   :safe #'consp)
 
-(defcustom a68-comment-style-supper "#"
-  "Default comment style used by e.g. `comment-dwim'."
-  :type '(choice (const "#")
-                 (const "co")
-                 (const "comment"))
-  :safe #'consp)
+;;;; Our own comment quoting function.
+
+(defun a68--comment-quote-nested (cs ce unp)
+  "Quote comment delimiters in the buffer for a68-mode."
+  (if (or (equal cs "{") (equal cs "NOTE"))
+      ;; Nestable comments do not require any extra handling on top of
+      ;; comment-region-default when unp is nil.  Non-nestable
+      ;; coments, used in the classic stropping regime, are all
+      ;; supposed to be properly handled by
+      ;; comment-quote-nested-default, with the exception of # .. #
+      ;; which is mishandled since it uses symmetrical delimiters of
+      ;; size one character where replacing # by #/ obviously doesn't
+      ;; have the right effect.  To improve.
+      (when unp
+        ;; Remove the end comment.
+        (goto-char (- (point-max) (length ce)))
+        (delete-char (- (length ce) 1)))
+    (comment-quote-nested-default cs ce unp)))
 
 ;;;; Syntax table for the a68-mode.
 
@@ -117,6 +143,9 @@
     (modify-syntax-entry ?, "." st)
     (modify-syntax-entry ?: "." st)
     (modify-syntax-entry ?_ "w" st)
+    ;; Note { and } are nestable.
+    (modify-syntax-entry ?{ "< n" st)
+    (modify-syntax-entry ?} "> n" st)
     ;; define parentheses to match
     (modify-syntax-entry ?\( "()" st)
     (modify-syntax-entry ?\) ")(" st)
@@ -175,30 +204,56 @@
 
   ;; SUPPER stropping.
   (defconst a68-std-modes-supper
-    '("int" "real" "bool" "char" "format" "void"
-      "compl" "bits" "bytes" "string" "sema" "file" "channel")
+    '("proc" "flex" "int" "real" "bool" "char" "format" "void" "op"
+      "compl" "bits" "bytes" "string" "sema" "file" "channel" "ref"
+      "loc" "heap" "struct" "long" "short" "union")
     "List of Algol 68 standard modes in SUPPER stropping.")
 
+  (defconst a68-constants-supper
+    '("nil" "false" "true" "skip" "empty"))
+
   (defconst a68-keywords-supper
-    '("true" "false" "empty" "at"
+    '("empty" "at"
       "pr" "pragmat"
       "up" "down"
       "andth" "orel" "is" "isnt"
-      "long" "short" "ref" "loc" "heap" "struct" "flex" "proc"
-      "union" "op" "prio" "mode" "begin" "end" "exit" "par" "if"
+      "prio" "mode" "begin" "end" "exit" "par" "if"
       "then" "elif" "else" "fi" "case" "in" "ouse" "out" "esac"
-      "nil" "of" "go" "goto" "skip" "for" "from" "by" "to" "while"
+      "of" "go" "goto" "for" "from" "by" "to" "while"
+      "module" "def" "fed" "postlude" "access" "pub"
       "do" "od" "unsafe" "assert")
     "List of Algol 68 keywords in SUPPER stropping."))
 
 ;;;; Font-lock keywords.
 
+(defun a68--string-matcher (re end)
+  "Search for a regexp within Algol 68 strings."
+  (let (pos (case-fold-search t))
+    (while (and (setq pos (re-search-forward re end t))
+                (not (a68-within-string))))
+    pos))
+
+(defun a68--string-break-matcher (end)
+  (a68--string-matcher "\\('[nrft']\\)\\|\\('(.*?)\\)" end))
+
+(defun a68--bad-string-break-matcher (end)
+  (a68--string-matcher "\\(''\\|[^']\\)\\('[^nrft'(]\\)" end))
+
 (defconst a68-font-lock-keywords-common
   (list
+   ;; Radix in bits denotations.
+   '("\\(\\(4\\|8\\|16\\)r\\)[ \t]*[0-9a-f]+" 1 ''a68-bits-radix-face)
+   ;; Digits in bits denotations.
+   '("\\(\\(4\\|8\\|16\\)r\\)[ \t]*\\([0-9a-f]+\\)" 3 ''a68-numbers-face)
+   ;; Digits in binary bits denotations.
+   '("\\<2r[ \t]*\\([01]+\\)" 1 ''a68-numbers-face)
+   ;; Binary bits denotations also highlight set bits.
+   '("\\<\\(2r\\)[ \t]*[01]+" (1 ''a68-bits-radix-face)
+     ("1" (re-search-backward "2r" nil t) nil (0 ''a68-bits-flip-face)))
    ;; String breaks.  Apostrophe is not (currently) a worthy character
    ;; out of strings, so for now we can just match it anywhere.
-   '("\\('[nrft']\\)\\|\\('(.*?)\\)" 0 ''a68-string-break-face prepend)
-   '("\\(''\\|[^']\\)\\('[^nrft'(]\\)" 2 ''font-lock-warning-face prepend)
+   '(a68--string-break-matcher 0 'a68-string-break-face t)
+   '(a68--bad-string-break-matcher 2 'font-lock-warning-face t)
    ;; Two or more consecutive underscore characters are always
    ;; illegal in this stropping regime.
    (cons "_[_]+" ''font-lock-warning-face))
@@ -238,18 +293,24 @@
               (eval `(or ,@a68-std-modes-supper))
               word-end)
           ''font-lock-type-face)
+    (cons (rx word-start
+              (eval `(or ,@a68-constants-supper))
+              word-end)
+          ''font-lock-constant-face)
     (cons (rx word-start
               (or "true" "false")
               word-end)
           ''font-lock-constant-face)
+    ;; Numbers.
+    (cons "\\<\\([0-9][0-9.]*\\)\\>" ''a68-numbers-face)
     ;; Tags.
     (cons "\\<\\([a-z][a-z]+_?\\)+\\>" ''font-lock-variable-name-face)
     ;; Mode names start with an upper case letter.
     ;; To distinguish from operator indications in highlighting,
     ;; we mandate type faced strings to have at least one
     ;; lower-case letter.
-    (cons "\\<\\([A-Z][A-Za-z_]*[a-z][A-Za-z_]*\\)\\>" ''font-lock-type-face)
-    (cons "\\<\\([A-Z][A-Z_]*\\)\\>" ''font-lock-keyword-face)))
+    (cons "\\<\\([A-Z][A-Za-z0-9_]*[a-z][A-Za-z0-9_]*\\)\\>" ''font-lock-type-face)
+    (cons "\\<\\([A-Z][A-Z0-9_]*\\)\\>" ''a68-operator-face)))
   "Highlighting expressions for Algol 68 mode in SUPPER stropping.")
 
 ;;;; Syntax-based text properties.
@@ -266,6 +327,13 @@
        (2 (when (not (a68-within-string)) (string-to-syntax ">")))
        (0 (ignore (put-text-property (match-beginning 0) (match-end 0)
                                      'syntax-multiline t))))
+      ((rx bow (group "N") "OTE" eow
+           (*? anychar)
+           bow "ETO" (group "N") eow)
+       (1 (when (not (a68-within-string)) (string-to-syntax "< bn")))
+       (2 (when (not (a68-within-string)) (string-to-syntax "> bn")))
+       (0 (ignore (put-text-property (match-beginning 0) (match-end 0)
+                                     'syntax-multiline t))))
       ((rx bow (group "C") "OMMENT" eow
            (*? anychar)
            bow "COMMEN" (group "T") eow)
@@ -282,34 +350,6 @@
                                      'syntax-multiline t)))))
      (point) end)))
 
-(defun a68-syntax-propertize-function-supper (start end)
-  (let ((case-fold-search nil))
-    (goto-char start)
-    (funcall
-     (syntax-propertize-rules
-      ((rx (group "#")
-           (*? anychar)
-           (group "#"))
-       (1 (when (not (a68-within-string)) (string-to-syntax "<")))
-       (2 (when (not (a68-within-string)) (string-to-syntax ">")))
-       (0 (ignore (put-text-property (match-beginning 0) (match-end 0)
-                                     'syntax-multiline t))))
-      ((rx bow (group "c") "omment" eow
-           (*? anychar)
-           bow "commen" (group "t") eow)
-       (1 (when (not (a68-within-string)) (string-to-syntax "< b")))
-       (2 (when (not (a68-within-string)) (string-to-syntax "> b")))
-       (0 (ignore (put-text-property (match-beginning 0) (match-end 0)
-                                     'syntax-multiline t))))
-      ((rx bow (group "c") "o" eow
-           (*? anychar)
-           bow "c" (group "o") eow)
-       (1 (when (not (a68-within-string)) (string-to-syntax "< c")))
-       (2 (when (not (a68-within-string)) (string-to-syntax "> c")))
-       (0 (ignore (put-text-property (match-beginning 0) (match-end 0)
-                                     'syntax-multiline t)))))
-     (point) end)))
-
 ;;;; Functions to move up and down a procedure.
 
 (defun a68-beginning-of-defun-upper (&optional count)
@@ -353,6 +393,7 @@ with the equivalent upcased form."
 
 (defconst a68--bnf-grammar
   '((id)
+    (declarer)
     (ids (id "-anchor-" id))
     (fields (fields "," fields)
             (ids))
@@ -368,16 +409,9 @@ with the equivalent upcased form."
     (exp (ids)
          (exp "of" exp)
          (exp "[" exp "]")
-         ("(" exp ")")
-         ("begin" exp "end")
-         ("module" exp "def" exp "fed")
+         ("module" exp "def"  exp "fed")
          ("module" exp "def" exp "postlude" exp "fed"))
     ;; Declarations:
-    (declaration (type-decl)
-                 (proc-decl))
-    (type-decl ("mode" type-decl*))
-    (type-decl* (type-decl* "," type-decl*)
-                (id "=" type-decl**))
     (type-decl** ("struct" args)
                  ("union" args)
                  ("proc" args))
@@ -385,8 +419,213 @@ with the equivalent upcased form."
              ("op" ids "=" args ids ":" exp))
     (proc-decl (proc-decl "," proc-decl)
                ("op" ids "=" args ids ":" exp)
-               ("proc" ids "=" ids ":" exp))
-    ;; Units:
+               ("proc" ids "=" ids ":" exp)
+               ("pub" "-proc-" ids "=" ids ":" exp))
+    ;; Compilation inputs
+    ;; ==================
+    (compilation-input (labeled-enclosed-clause)
+                       (module-declaration))
+    (labeled-enclosed-clause ("-label-" labeled-enclosed-clause)
+                              ("-label-" enclosed-clause))
+    ;; Clauses
+    ;; =======
+    (enclosed-clause (closed-clause)
+                     (choice-clause)
+                     (loop-clause)
+                     (access-clause))
+    ;; Access clause
+    ;; -------------
+    ;;   access clause :
+    ;;     revelation, invoke insert, enclosed clause.
+    ;;   revelation :
+    ;;     access token, joined module call, ssecca insert.
+    ;;   joined module call :
+    ;;     module call, (separate and also token, joined module call).
+    ;;   module call :
+    ;;     (public token), invocation.
+    ;;   invocation :
+    ;;     module indication.
+    (access-clause ("access" joined-module-call "-ssecca-" enclosed-clause))
+    (joined-module-call (joined-module-call "," joined-module-call)
+                        (invocation)
+                        ("pub" invocation))
+    (invocation ("-bold-"))
+    ;; Closed clause
+    ;; -------------
+    ;;   closed or collateral clause :
+    ;;     begin, inner clause, end.
+    ;;   begin :
+    ;;     bold begin token ; brief begin token.
+    ;;   end :
+    ;;     bold end token ; brief end token.
+    ;;   inner clause :
+    ;;     serial clause ;
+    ;;     (joined portrait).
+    (closed-clause ("begin" serial "end")
+                   ("(" serial ")"))
+    ;; Serial clause
+    ;; -------------
+    ;;   serial clause :
+    ;;     series.
+    ;;   series :
+    ;;     train, (completion token, series).
+    ;;   train :
+    ;;     declun, go on token, train ; lunit.
+    ;;   declun :
+    ;;     declaration ; lunit.
+    ;;   lunit :
+    ;;     label definition, lunit ; unit.
+    ;;   label definition :
+    ;;     identifier, label token.
+    (pragmat ("-pr-" exp "pr"))
+    (serial (serial ";" serial)
+            (unit)
+            (module)
+            ("-label-" unit)
+            (declaration)
+            (pragmat))
+    ;; Enquiry clause
+    ;; --------------
+    ;;  enquiry clause :
+    ;;   series.
+    (enquiry-clause (serial))
+    ;; Choice clauses
+    ;;   choice clause :
+    ;;     choice start, chooser choice clause, choice finish.
+    ;;   chooser choice clause :
+    ;;     enquiry clause, alternate choice clause.
+    ;;   enquiry clause :
+    ;;     series.
+    ;;   alternate choice clause :
+    ;;     in choice clause, (out choice clause).
+    ;;   in choice clause :
+    ;;     choice in, in part of choice.
+    ;;   in part of choice :
+    ;;     serial clause ; case part list proper ; united case part.
+    ;;   case part list proper :
+    ;;     case part list, and also token, case part.
+    ;;   case part list :
+    ;;     (case part list, and also token), case part.
+    ;;   case part :
+    ;;     unit ; united case part.
+    ;;   united case part :
+    ;;     specification, unit.
+    ;;   specification :
+    ;;     single declaration brief pack, specification token.
+    ;;   single declaration brief pack :
+    ;;     brief begin token, single declaration, brief end token.
+    ;;   single declaration :
+    ;;     declarer, (dectag insert, identifier).
+    ;;   out choice clause :
+    ;;     choice out, serial clause ;
+    ;;     choice again, chooser choice clause.
+    (choice-clause ("if" enquiry-clause "then" serial "fi")
+                   ("if" enquiry-clause "then" serial "else" serial "fi")
+                   ("if" enquiry-clause "then" serial
+                    "elif" enquiry-clause "then" serial "fi")
+                   ("(" enquiry-clause "|" serial ")")
+                   ("(" enquiry-clause "|" serial "|" serial ")")
+                   ("(" enquiry-clause "|" serial
+                    "|:" enquiry-clause "|" serial ")")
+                   ("case" enquiry-clause "in" specs "esac")
+                   ("case" enquiry-clause "in" specs "out" serial "esac")
+                   ("case" enquiry-clause "in" specs "ouse" enquiry-clause "in" specs "esac")
+                   ("case" enquiry-clause "in" specs "ouse" enquiry-clause "in" specs "out" serial "esac")
+                   ("(" enquiry-clause "|" specs ")")
+                   ("(" enquiry-clause "|" specs "|" serial ")")
+                   ("(" enquiry-clause "|" specs "|:" enquiry-clause "|" specs ")")
+                   ("(" enquiry-clause "|" specs "|:" enquiry-clause "|" specs "|" serial ")"))
+    ;; Loop clauses.
+    ;;   loop clause :
+    ;;     loop insert, for part, (from part), (by part), (to part), repeating part.
+    ;;   for part :
+    ;;     (for token, identifier).
+    ;;   from part :
+    ;;     from token, unit.
+    ;;   by part :
+    ;;     by token, unit.
+    ;;   to part :
+    ;;     to token, unit.
+    ;;   repeating part :
+    ;;     (while part), do part.
+    ;;   while part :
+    ;;     while token, enquiry clause.
+    ;;   do part :
+    ;;     do token, serial clause, od token.
+    (loop-clause ("for" id "do" serial "od")
+                 ("for" id "from" serial "do" serial "od")
+                 ("for" id "from" serial "by" serial "do" serial "od")
+                 ("for" id "from" serial "by" serial "to" serial "do" serial "od")
+                 ("for" id "from" serial "by" serial "to" serial "while" serial "do" serial "od")
+                 ("-from-" serial "by" serial "to" serial "while" serial "do" serial "od")
+                 ("-from-" serial "by" serial "to" serial "do" serial "od")
+                 ("-from-" serial "by" serial "do" serial "od")
+                 ("-from-" serial "do" serial "od")
+                 ("-by-" serial "to" serial "while" serial "do" serial "od")
+                 ("-by-" serial "while" serial "do" serial "od")
+                 ("-by-" serial "do" serial "od")
+                 ("-to-" serial "while" serial "do" serial "od")
+                 ("-to-" serial "do" serial "od")
+                 ("-while-" serial "do" serial "od")
+                 ("-do-" serial "od"))
+    ;; Declarations
+    ;; ============
+    ;;   declaration :
+    ;;     publety ldecety declaration,
+    ;;       (separate and also token, declaration).
+    ;;   publety ldecety declaration :
+    ;;     (public token), ldecety declaration.
+    ;;   ldecety declaration :
+    ;;     (ldec token), common declaration.
+    ;;   common declaration :
+    ;;     mode declaration ; priority declaration ;
+    ;;     identifier declaration ; operation declaration ;
+    ;;     module declaration.
+    (declaration ("pub" declaration*)
+                 (declaration*))
+    (declaration* (mode-declaration)
+                  (priority-declaration)
+                  (identifier-declaration)
+                  (operation-declaration)
+                  (module-declaration))
+    ;; Mode declarations
+    ;; -----------------
+    ;;   mode declaration :
+    ;;     mode token, mode joined definition.
+    ;;   mode joined definition :
+    ;;     (mode joined definition, and also token), mode definition.
+    ;;   mode definition :
+    ;;     defined mode indication, is defined as token, declarer or code.
+    ;;   defined mode indication :
+    ;;     mode indication.
+    ;;   declarer or code :
+    ;;     declarer ; code.
+    (mode-declaration ("mode" mode-joined-definition))
+    (mode-joined-definition (mode-joined-definition "," mode-joined-definition)
+                            (mode-definition))
+    (mode-definition (mode-indication "=" declarer-or-code))
+    (mode-indication ("-bold-"))
+    (declarer-or-code (declarer)
+                      (code))
+    ;; Priority declarations
+    ;; ---------------------
+    ;;   priority declaration :
+    ;;     priority token, priority joined definition.
+    ;;   priority joined definition :
+    ;;     (priority joined definition, and also token), priority definition.
+    ;;   priority definition :
+    ;;     operator, is defined as token, priority unit.
+    ;;   priority unit :
+    ;;     digit token.
+    (priority-declaration ("prio" priority-joined-definition))
+    (priority-joined-definition (priority-joined-definition "," priority-joined-definition)
+                                (priority-definition))
+    (priority-definition (operator "=" priority-unit))
+    (operator ("-oper-"))
+    (priority-unit ("1") ("2") ("3") ("4") ("5")
+                   ("6") ("7") ("8") ("9"))
+    ;; Units
+    ;; =====
     ;;  unit :
     ;;    assignation ; identity relation ; routine text ;
     ;;    function and ; function or ; tertiary.
@@ -399,11 +638,10 @@ with the equivalent upcased form."
     ;;  primary one :
     ;;    slice call ; cast ; string denoter ; identifier ;
     ;;    jump ; enclosed clause.
-    (unit (id ":=" exp)
-          ; (routine-text)
+    (unit ; (routine-text)
           (assignation)
-          (pseudo-operator))
-    (assignation (tertiary ":=" unit))
+          (pseudo-operator)
+          (tertiary))
     (tertiary (formula)
               (secondary))
     (secondary (leap-geneator)
@@ -420,22 +658,34 @@ with the equivalent upcased form."
                  (id)
                  (jump)
                  (enclosed-clause))
-    (jump ("goto" id)
-          ("go" "-to-jump-" id))
-    (pseudo-operator (exp "andth" exp)
-                     (exp "orel" exp)
-                     (exp ":=:" exp)
-                     (exp ":/=:" exp)
-                     (exp "is" exp)
-                     (exp "isnt" exp))
+    ;; Assignations
+    ;; ------------
+    (assignation (tertiary ":=" unit))
+    ;; Pseudo-operators
+    ;; ----------------
+    (pseudo-operator (tertiary "andth" tertiary)
+                     (tertiary "orel" tertiary)
+                     (tertiary ":=:" tertiary)
+                     (tertiary ":/=:" tertiary)
+                     (tertiary "is" tertiary)
+                     (tertiary "isnt" tertiary))
+    ;; Generators
+    ;; ----------
+    (leap-generator ("heap" declarer)
+                    ("loc" declarer))
+    ;; Selections
+    ;; ----------
     (selection (id "of" secondary))
-    ;; Formula.
-    ;; Standard operators are given their priority.
+    ;; Slices
+    ;; ------
+    ;; XXX
+    ;; Routine texts
+    ;; -------------
+    ;; XXX
+    ;; Formulas
+    ;; --------
     (formula (dyadic-formula)
              (monadic-formula))
-    (monadic-formula ("-monadic~-" monadic-operand)
-                     ("-monadic+-" monadic-operand)
-                     ("-monadic--" monadic-operand))
     (dyadic-formula (operand "-oper-" monadic-operand)
                     (operand "+:=" monadic-operand)
                     (operand "-:=" monadic-operand)
@@ -486,100 +736,20 @@ with the equivalent upcased form."
                     (operand "I" monadic-operand)
                     (operand "+*" monadic-operand)
                     (operand "ELEMS" monadic-operand))
+    (monadic-formula ("-monadic~-" monadic-operand)
+                     ("-monadic+-" monadic-operand)
+                     ("-monadic--" monadic-operand))
     (operand (formula)
              (secondary))
     (monadic-operand (monadic-formula)
                      (secondary))
-    ;; Enquiry clause:
-    ;;  enquiry clause :
-    ;;   series.
-    (enquiry-clause (serial))
-    ;; Clauses:
-    (enclosed-clause (choice-clause)
-                     (loop-clause))
-    ;; Choice clauses
-    ;;   choice clause :
-    ;;     choice start, chooser choice clause, choice finish.
-    ;;   chooser choice clause :
-    ;;     enquiry clause, alternate choice clause.
-    ;;   enquiry clause :
-    ;;     series.
-    ;;   alternate choice clause :
-    ;;     in choice clause, (out choice clause).
-    ;;   in choice clause :
-    ;;     choice in, in part of choice.
-    ;;   in part of choice :
-    ;;     serial clause ; case part list proper ; united case part.
-    ;;   case part list proper :
-    ;;     case part list, and also token, case part.
-    ;;   case part list :
-    ;;     (case part list, and also token), case part.
-    ;;   case part :
-    ;;     unit ; united case part.
-    ;;   united case part :
-    ;;     specification, unit.
-    ;;   specification :
-    ;;     single declaration brief pack, specification token.
-    ;;   single declaration brief pack :
-    ;;     brief begin token, single declaration, brief end token.
-    ;;   single declaration :
-    ;;     declarer, (dectag insert, identifier).
-    ;;   out choice clause :
-    ;;     choice out, serial clause ;
-    ;;     choice again, chooser choice clause.
-    (choice-clause ("if" enquiry-clause "then" serial "fi")
-                   ("if" enquiry-clause "then" serial "else" serial "fi")
-                   ("if" enquiry-clause "then" serial
-                    "elif" enquiry-clause "then" serial "fi")
-                   ("(" enquiry-clause "|" serial ")")
-                   ("(" enquiry-clause "|" serial "|" serial ")")
-                   ("(" enquiry-clause "|" serial
-                    "|:" enquiry-clause "|" serial ")")
-                   ("case" enquiry-clause "in" specs "esac")
-                   ("case" enquiry-clause "in" specs "out" serial "esac")
-                   ("case" enquiry-clause "in" specs "ouse" serial "esac")
-                   ("(" enquiry-clause "|" specs ")")
-                   ("(" enquiry-clause "|" specs "|" serial ")")
-                   ("(" enquiry-clause "|" specs "|:" serial ")"))
-    ;; Loop clauses.
-    ;;   loop clause :
-    ;;     loop insert, for part, (from part), (by part), (to part), repeating part.
-    ;;   for part :
-    ;;     (for token, identifier).
-    ;;   from part :
-    ;;     from token, unit.
-    ;;   by part :
-    ;;     by token, unit.
-    ;;   to part :
-    ;;     to token, unit.
-    ;;   repeating part :
-    ;;     (while part), do part.
-    ;;   while part :
-    ;;     while token, enquiry clause.
-    ;;   do part :
-    ;;     do token, serial clause, od token.
-    (loop-clause ("for" id "do" serial "od")
-                 ("for" id "from" serial "do" serial "od")
-                 ("for" id "from" serial "by" serial "do" serial "od")
-                 ("for" id "from" serial "by" serial "to" serial "do" serial "od")
-                 ("for" id "from" serial "by" serial "to" serial "while" serial "do" serial "od")
-                 ("-from-" serial "by" serial "to" serial "while" serial "do" serial "od")
-                 ("-from-" serial "by" serial "to" serial "do" serial "od")
-                 ("-from-" serial "by" serial "do" serial "od")
-                 ("-from-" serial "do" serial "od")
-                 ("-by-" serial "to" serial "while" serial "do" serial "od")
-                 ("-by-" serial "while" serial "do" serial "od")
-                 ("-by-" serial "do" serial "od")
-                 ("-to-" serial "while" serial "do" serial "od")
-                 ("-to-" serial "do" serial "od")
-                 ("-while-" serial "do" serial "od")
-                 ("-do-" serial "od"))
-    (pragmat ("-pr-" exp "pr"))
-    (serial (serial ";" serial)
-            (unit)
-            ("-label-" unit)
-            (declaration)
-            (pragmat)))
+    ;; Jumps
+    ;; -----
+    (jump ("goto" id)
+          ("go" "-to-jump-" id))
+    ;; Casts
+    ;; -----
+    (cast (declarer "-cast-" enclosed-clause)))
   "Algol 68 BNF operator precedence grammar to use with SMIE")
 
 (defvar a68--smie-grammar-upper
@@ -606,6 +776,26 @@ with the equivalent upcased form."
 (defvar a68--keywords-regexp
   (regexp-opt '("|:" "(" ")" "+" "*" ";" ">" "<" ":=" "=" "," ":" "~")))
 
+(defconst a68--monads
+  '("%" "^" "&" "+" "-" "~" "!" "?"))
+
+(defconst a68--nomads
+  '(">" "<" "/" "=" "*"))
+
+;; An operator indication is:
+;; - A bold tag, or.
+;; - Any monad, or
+;; - A monad followed by a nomad, or
+;; - A monad optionally followed by a nomad followd by either
+;;   := or =:, but not by both.
+
+(defvar a68--oper-regexp
+  (concat "\\(?:"
+          (regexp-opt a68--monads)
+          (regexp-opt a68--nomads) "?"
+          "\\(?::=\\|=:\\)?"
+          "\\)"))
+
 (defun a68-at-strong-void-enclosed-clause-supper ()
   "Return whether the point is at the beginning of a VOID enclosed clause."
   (save-excursion
@@ -622,7 +812,7 @@ with the equivalent upcased form."
                                  "while" "do" "(" "|" "|:" "def" "postlude"))
                    (- (point) 8))
      ;; tag denotation or mode indication
-     (and (looking-back "[A-Z][A-Za-z_]+" (pos-bol))
+     (and (looking-back "[A-Z][A-Za-z0-9_]+" (pos-bol))
           ;; Given the context at hand, i.e. a bold word followed
           ;; by "from", "to", "by", "while" or "do", we are at the
           ;; beginning of an enclosed clause if we are part of:
@@ -669,7 +859,7 @@ with the equivalent upcased form."
         ;; mode-indication consists of the symbols "loc" and "heap",
         ;; plus those symbols which may immediately precede a
         ;; mode-indication in an actual-MODE-declarer.
-        (or (looking-back "[A-Z][A-Za-z_]+" (pos-bol))
+        (or (looking-back "[A-Z][A-Za-z0-9_]+" (pos-bol))
             (looking-back (regexp-opt '("loc" "heap"
                                         "ref" ")" "]"
                                         "proc" "flex"))
@@ -679,6 +869,21 @@ with the equivalent upcased form."
   (forward-comment (point-max))
   (let ((case-fold-search nil))
     (cond
+     ;; operator.
+     ((looking-at a68--oper-regexp)
+      (goto-char (match-end 0))
+      "-oper-")
+     ;; A bold-word may be a ssecca insert if it is preceded by a
+     ;; joined list of bold words, preceded by access.
+     ((looking-at "[A-Z][A-Za-z0-9_]+")
+      (let* ((end (match-end 0))
+             (token (if (and (not (looking-at "[A-Z][A-Za-z0-9_]+[ \t\n]*,"))
+                             (looking-back "access[ \t\n]*\\([ \t\n]*[A-Z][A-Za-z0-9_]+[ \t\n]*,\\)*[ \t\n]*"
+                                           nil))
+                        "-ssecca-"
+                      "-bold-")))
+        (goto-char end)
+        token))
      ;; A semicolon following a tag is a label, but mind standard modes.
      ((and (looking-at "\\<[a-z]+:")
            (let ((beg (match-beginning 0))
@@ -706,6 +911,15 @@ with the equivalent upcased form."
       (if (looking-at "[ \t\n]*\\<include\\>")
           "-pr-"
         "pr"))
+     ;; A -proc- follows pub.
+     ((looking-at "\\<proc\\>")
+      (cond
+       ((looking-back "\\<pub\\>[ \t\n]*" nil)
+        (goto-char (+ (point) 4))
+        "-proc-")
+       (t
+        (goto-char (+ (point) 4))
+        "proc")))
      ;; The symbols "by", "from", "to", "while" and "do" mark the start
      ;; of a loop-clause if they are the first symbol of an
      ;; enclosed-clause, and is thus preceded by a symbol which may
@@ -790,6 +1004,18 @@ with the equivalent upcased form."
   (forward-comment (- (point)))
   (let ((case-fold-search nil))
     (cond
+     ;; operator, so any nomad or monad.
+     ((looking-back a68--oper-regexp
+                    (pos-bol))
+      (goto-char (match-beginning 0))
+      "-oper-")
+     ((looking-back "[A-Z][A-Za-z0-9_]+" (pos-bol))
+      (goto-char (match-beginning 0))
+      (if (and (not (looking-at "[A-Z][A-Za-z0-9_]+[ \t\n]*,"))
+               (looking-back "access[ \t\n]*\\([ \t\n]*[A-Z][A-Za-z0-9_]+[ \t\n]*,\\)*[ \t\n]*"
+                             nil))
+          "-ssecca-"
+        "-bold-"))
      ((and (looking-back "\\<[a-z]+:" (pos-bol))
            (let ((beg (match-beginning 0))
                  (end (match-end 0)))
@@ -812,6 +1038,14 @@ with the equivalent upcased form."
      ((looking-back "):" (- (point) 2))
       (goto-char (- (point) 2))
       "):")
+     ;; A -proc- follows pub.
+     ((looking-back "\\<proc\\>")
+      (goto-char (- (point) 4))
+      (cond
+       ((looking-back "\\<pub\\>[ \t\n]*" nil)
+        "-proc-")
+       (t
+        "proc")))
      ;; See comments in a68--smie-forward-token for an explanation of
      ;; the handling of loop insertions -from- -to- -by- -while-.
      ((looking-back "\\<from\\>" (- (point) 4))
@@ -945,6 +1179,17 @@ UPPER stropping version."
   (forward-comment (point-max))
   (let ((case-fold-search nil))
     (cond
+     ;; A bold-word may be a ssecca insert if it is preceded by a
+     ;; joined list of bold words, preceded by access.
+     ((looking-at "[A-Z][A-Z_]+")
+      (let* ((end (match-end 0))
+             (token (if (and (not (looking-at "[A-Z][A-Z_]+[ \t\n]*,"))
+                             (looking-back "access[ \t\n]*\\([ \t\n]*[A-Z][A-Z_]+[ \t\n]*,\\)*[ \t\n]*"
+                                           nil))
+                        "-ssecca-"
+                      "-bold-")))
+        (goto-char end)
+        token))
      ;; A semicolon following a tag is a label.
      ((looking-at "\\<[a-z]+:")
       (goto-char (match-end 0))
@@ -967,6 +1212,15 @@ UPPER stropping version."
       (if (looking-at "[ \t\n]*\\<include\\>")
           "-pr-"
         "PR"))
+     ;; A -proc- follows pub.
+     ((looking-at "\\<PROC\\>")
+      (cond
+       ((looking-back "\\<PUB\\>[ \t\n]*" nil)
+        (goto-char (+ (point) 4))
+        "-proc-")
+       (t
+        (goto-char (+ (point) 4))
+        "PROC")))
      ;; The symbols "by", "from", "to", "while" and "do" mark the
      ;; start of a loop-clause if they are the first symbol of an
      ;; enclosed-clause, and is thus preceded by a symbol which may
@@ -1051,6 +1305,13 @@ UPPER stropping version."
   (forward-comment (- (point)))
   (let ((case-fold-search nil))
     (cond
+     ((looking-back "[A-Z][A-Z]+" (pos-bol))
+      (goto-char (match-beginning 0))
+      (if (and (not (looking-at "[A-Z][A-Z_]+[ \t\n]*,"))
+               (looking-back "access[ \t\n]*\\([ \t\n]*[A-Z][A-Z_]+[ \t\n]*,\\)*[ \t\n]*"
+                             nil))
+          "-ssecca-"
+        "-bold-"))
      ((looking-back "\\<[a-z]+:" (pos-bol))
       (goto-char (match-beginning 0))
       "-label-")
@@ -1068,6 +1329,14 @@ UPPER stropping version."
      ((looking-back "):" (- (point) 2))
       (goto-char (- (point) 2))
       "):")
+     ;; A -proc- follows pub.
+     ((looking-back "\\<PROC\\>")
+      (goto-char (- (point) 4))
+      (cond
+       ((looking-back "\\<PUB\\>[ \t\n]*" nil)
+        "-proc-")
+       (t
+        "PROC")))
      ;; See comments in a68--smie-forward-token for an explanation of
      ;; the handling of loop insertions -from- -to- -by- -while-.
      ((looking-back "\\<FROM\\>" (- (point) 4))
@@ -1137,7 +1406,7 @@ UPPER stropping version."
     ;; Since "|" is in the same BNF rule as "(" in choice-clauses,
     ;; SMIE by default aligns it with it.
     (`(:before . "|")
-     (if (not smie-rule-sibling-p) 3))
+     (if (not (smie-rule-sibling-p)) 3))
     (`(:after . "BEGIN") 6)
     (`(:after . "THEN") 5)
     (`(:after . "ELSE") 5)
@@ -1153,6 +1422,9 @@ UPPER stropping version."
     (`(:after . "TO") 3)
     (`(:after . "WHILE") 3)
     (`(:after . "DEF") 4)
+    (`(:after . "POSTLUDE") 4)
+    (`(:before . "-ssecca-")
+     (smie-rule-parent 7))
     (`(:before . "-label-")
      (smie-rule-parent))
     (`(:before . "BEGIN")
@@ -1161,7 +1433,8 @@ UPPER stropping version."
                 (and (or (smie-rule-parent-p "PROC")
                          (smie-rule-parent-p "OP"))
                      (smie-rule-prev-p ":"))
-                (smie-rule-parent-p "PROGRAM")))
+                (smie-rule-parent-p "PROGRAM")
+                (smie-rule-parent-p "ACCESS")))
        (smie-rule-parent)))
     (`(:before . "THEN")
      (when (or (smie-rule-hanging-p)
@@ -1198,15 +1471,20 @@ UPPER stropping version."
     (`(:after . "to") 3)
     (`(:after . "while") 3)
     (`(:after . "def") 4)
+    (`(:after . "postlude") 4)
+    (`(:before . "-ssecca-")
+     (smie-rule-parent 7))
     (`(:before . "-label-")
      (smie-rule-parent))
     (`(:before . "begin")
      (when (or (smie-rule-hanging-p)
                (or
-                (and (or (smie-rule-parent-p "proc")
+                (and (or (smie-rule-parent-p "pub")
+                         (smie-rule-parent-p "proc")
                          (smie-rule-parent-p "op"))
                      (smie-rule-prev-p ":"))
-                (smie-rule-parent-p "program")))
+                (smie-rule-parent-p "program")
+                (smie-rule-parent-p "access")))
        (smie-rule-parent)))
     (`(:before . "then")
      (when (or (smie-rule-hanging-p)
@@ -1243,17 +1521,18 @@ UPPER stropping version."
   ;; First determine the stropping regime
   (setq-local a68--stropping-regime
               (a68--figure-out-stropping-regime))
+  (setq-local comment-quote-nested-function #'a68--comment-quote-nested)
   (cond
    ((equal a68--stropping-regime 'supper)
     ;; SUPPER stropping.
-    (setq-local comment-start a68-comment-style-supper)
-    (setq-local comment-end a68-comment-style-supper)
     (setq-local font-lock-defaults '(a68-font-lock-keywords-supper))
     (smie-setup a68--smie-grammar-supper #'a68--smie-rules-supper
                 :forward-token #'a68--smie-forward-token-supper
                 :backward-token #'a68--smie-backward-token-supper)
     (setq-local beginning-of-defun-function #'a68-beginning-of-defun-supper)
-    (setq-local syntax-propertize-function #'a68-syntax-propertize-function-supper))
+    (setq-local comment-start "{")
+    (setq-local comment-start-skip "{ *")
+    (setq-local comment-end "}"))
    (t
     ;; UPPER stropping.
     (setq-local comment-start a68-comment-style-upper)
@@ -1263,10 +1542,10 @@ UPPER stropping version."
                 :forward-token #'a68--smie-forward-token-upper
                 :backward-token #'a68--smie-backward-token-upper)
     (setq-local beginning-of-defun-function #'a68-beginning-of-defun-upper)
-    (setq-local syntax-propertize-function #'a68-syntax-propertize-function-upper)))
-  (setq-local comment-start-skip "\\(#\\) *")
-  (add-hook 'syntax-propertize-extend-region-functions
-            #'syntax-propertize-multiline 'append 'local))
+    (setq-local comment-start-skip "\\(#\\) *")
+    (setq-local syntax-propertize-function #'a68-syntax-propertize-function-upper)
+    (add-hook 'syntax-propertize-extend-region-functions
+              #'syntax-propertize-multiline 'append 'local))))
 
 ;;;###autoload
 (add-to-list 'auto-mode-alist '("\\.a68\\'" . a68-mode))
